<?php

namespace App\Controller\Frontend;

use App\Form\SearchListForm;
use App\Repository\DreamStore;
use App\Repository\ProjectBoostStore;
use App\Repository\ProjectInspiringStore;
use App\Repository\RevoltStore;
use Doctrine\ORM\EntityManagerInterface;
use Dompdf\Dompdf;
use Dompdf\Options;
use Knp\Component\Pager\PaginatorInterface;
use Lc\SovBundle\Generator\CsvGenerator;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Twig\Environment;

class CartoController extends DefaultController
{
    protected EntityManagerInterface $em;
    protected DreamStore $dreamStore;
    protected RevoltStore $revoltStore;
    protected ProjectBoostStore $projectBoostStore;
    protected ProjectInspiringStore $projectInspiringStore;
    protected PaginatorInterface $paginator;
    protected Environment $templating;
    protected ParameterBagInterface $parameterBag;

    public function __construct(
        EntityManagerInterface $em,
        DreamStore $dreamStore,
        RevoltStore $revoltStore,
        ProjectBoostStore $projectBoostStore,
        ProjectInspiringStore $projectInspiringStore,
        PaginatorInterface $paginator,
        Environment $templating,
        ParameterBagInterface $parameterBag
    ) {
        $this->em = $em;
        $this->dreamStore = $dreamStore;
        $this->revoltStore = $revoltStore;
        $this->projectBoostStore = $projectBoostStore;
        $this->projectInspiringStore = $projectInspiringStore;
        $this->paginator = $paginator;
        $this->templating = $templating;
        $this->parameterBag = $parameterBag;
    }

    public function cartoInteractive()
    {
        return $this->render(
            'frontend/carto-int.html.twig',
            [
                'nbContrib' => $this->countContrib()
            ]
        );
    }

    public function cartoListe(Request $request)
    {
        $resultArrayPagination = array();

        $form = $this->createForm(SearchListForm::class);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $data = $form->getData();

            $resultArray = $this->generateResultArray($data);

            if ($form->get('search_button')->isClicked()) {
                $page = $data['page'];

                $resultArrayPagination = $this->paginator->paginate(
                    $resultArray,
                    $page,
                    10
                );
            } elseif ($form->get('export_excel')->isClicked()) {
                return $this->exportCsv($resultArray);
            } elseif ($form->get('export_pdf')->isClicked()) {
                return $this->exportPdf($resultArray);
            }
        }

        return $this->render(
            'frontend/carto-liste.html.twig',
            [
                'form' => $form->createView(),
                'nbContrib' => $this->countContrib(),
                'resultArray' => $resultArrayPagination
            ]
        );
    }

    private function generateResultArray($data): array
    {
        $dreamArray = $revoltArray = $projectBoostArray = $projectInspiringArray = array();

        $subthematic = $data['search'];
        $categoryArray = $data['category'];
        $territoryArray = $data['territory'];
        $thematicArray = $data['thematic'];

        if (in_array('dream', $categoryArray) || empty($categoryArray)) {
            $dreamArray = $this->dreamStore->filterSearch($subthematic, $territoryArray, $thematicArray);
        }
        if (in_array('revolt', $categoryArray) || empty($categoryArray)) {
            $revoltArray = $this->revoltStore->filterSearch($subthematic, $territoryArray, $thematicArray);
        }
        if (in_array('projectBoost', $categoryArray) || empty($categoryArray)) {
            $projectBoostArray = $this->projectBoostStore->filterSearch(
                $subthematic,
                $territoryArray,
                $thematicArray
            );
        }
        if (in_array('projectInspiring', $categoryArray) || empty($categoryArray)) {
            $projectInspiringArray = $this->projectInspiringStore->filterSearch(
                $subthematic,
                $territoryArray,
                $thematicArray
            );
        }

        return array_merge($dreamArray, $revoltArray, $projectBoostArray, $projectInspiringArray);
    }

    public function cartoCarte()
    {
        return $this->render(
            'frontend/carto-carte.html.twig',
            [
                'nbContrib' => $this->countContrib()
            ]
        );
    }

    public function exportPdf($resultArray = array())
    {
        // Configure Dompdf according to your needs
        $pdfOptions = new Options();
        $pdfOptions->set('defaultFont', 'Arial');

        // Instantiate Dompdf with our options
        $dompdf = new Dompdf($pdfOptions);

        if (empty($resultArray)) {
            $resultArray = $this->generateAllResultArray();
        }

        // Retrieve the HTML generated in our twig
        $html = $this->templating->render('frontend/pdf.html.twig', [
            'resultArray' => $resultArray,
            'css' => file_get_contents(
                $this->parameterBag->get('app.assets_directory') . 'css/pdf.css'
            )
        ]);

//        return new Response($html);
        // Load HTML to Dompdf
        $dompdf->loadHtml($html);

        // (Optional) Setup the paper size and orientation 'portrait' or 'portrait'
        $dompdf->setPaper('A4', 'portrait');


        // Render the HTML as PDF
        $dompdf->render();
        $dompdf->stream('Export-pdf.pdf', [
            "Attachment" => true
        ]);
    }

    public function exportCsv($resultArray = array())
    {
        $csv = new CsvGenerator();
        $csv->enableConvertEncoding('ISO-8859-1');
        $csv->setTitle('Export_Liste', true);

        $columns = [
            'category' => 'Catégorie',
            'thematic' => 'Thématique',
            'subthematic' => 'Contribution',
            'territory' => 'Lieu'
        ];
        $csv->setColumns($columns);

        if (empty($resultArray)) {
            $resultArray = $this->generateAllResultArray();
        }

        $csv = $this->generateCsvData($csv, $resultArray);

        return $csv->getReponse();
    }

    private function generateAllResultArray(): array
    {
        $dreamArray = $this->dreamStore->get();
        $revoltArray = $this->revoltStore->get();
        $projectBoostArray = $this->projectBoostStore->get();
        $projectInspiringArray = $this->projectInspiringStore->get();

        return array_merge($dreamArray, $revoltArray, $projectBoostArray, $projectInspiringArray);
    }

    private function generateCsvData($csv, $resultArray)
    {
        foreach ($resultArray as $result) {
            $territory = $subthematic = $thematic = "";
            if ($result->getIndividualData()) {
                $territory = $result->getIndividualData()->getTerritory()->getName();
            } elseif ($result->getCollectifData()->getTerritory()) {
                $territory = $result->getCollectifData()->getTerritory()->getName();
            }

            if ($result->getSubthematic()) {
                $subthematic = $result->getSubthematic()->getName();
            }

            if ($result->getThematic()) {
                $thematic = $result->getThematic()->getName();
            }

            $data = [
                'category' => $result->__toString(),
                'thematic' => $thematic,
                'subthematic' => $subthematic,
                'territory' => $territory
            ];

            $csv->row($data);
        }

        return $csv;
    }
}